import { FunctionValueGenerator, ValueGenerator } from "./functions/ValueGenerator";
import { Behavior } from "./behaviors/Behavior";
import { Particle } from "./Particle";
import { MetaData, ParticleEmitter } from "./ParticleEmitter";
import { EmitterShape, ShapeJSON } from "./shape/EmitterShape";
import { Blending, BufferGeometry, Matrix4, Texture } from "three";
import { ColorGenerator, FunctionColorGenerator, FunctionJSON } from "./functions";
import { ParticleSystemBatchSettings, RenderMode } from "./ParticleSystemBatch";
import { BatchedParticleRenderer } from "./BatchedParticleRenderer";
export interface BurstParameters {
    time: number;
    count: number;
    cycle: number;
    interval: number;
    probability: number;
}
export interface ParticleSystemParameters {
    autoDestroy?: boolean;
    looping?: boolean;
    duration?: number;
    maxParticle?: number;
    shape?: EmitterShape;
    startLife?: ValueGenerator | FunctionValueGenerator;
    startSpeed?: ValueGenerator | FunctionValueGenerator;
    startRotation?: ValueGenerator | FunctionValueGenerator;
    startSize?: ValueGenerator | FunctionValueGenerator;
    startLength?: ValueGenerator | FunctionValueGenerator;
    startColor?: ColorGenerator | FunctionColorGenerator;
    emissionOverTime?: ValueGenerator | FunctionValueGenerator;
    emissionOverDistance?: ValueGenerator | FunctionValueGenerator;
    emissionBursts?: Array<BurstParameters>;
    onlyUsedByOther?: boolean;
    behaviors?: Array<Behavior>;
    instancingGeometry?: BufferGeometry;
    renderMode?: RenderMode;
    rendererEmitterSettings?: TrailSettings | MeshSettings | BillBoardSettings;
    speedFactor?: number;
    texture: Texture;
    startTileIndex?: ValueGenerator;
    uTileCount?: number;
    vTileCount?: number;
    renderOrder?: number;
    blending?: Blending;
    transparent?: boolean;
    worldSpace?: boolean;
}
export interface ParticleSystemJSONParameters {
    autoDestroy: boolean;
    looping: boolean;
    duration: number;
    maxParticle: number;
    shape: ShapeJSON;
    startLife: FunctionJSON;
    startSpeed: FunctionJSON;
    startRotation: FunctionJSON;
    startSize: FunctionJSON;
    startColor: FunctionJSON;
    emissionOverTime: FunctionJSON;
    emissionOverDistance: FunctionJSON;
    emissionBursts?: Array<BurstParameters>;
    onlyUsedByOther: boolean;
    rendererEmitterSettings: {
        startLength?: FunctionJSON;
        followLocalOrigin?: boolean;
    };
    instancingGeometry?: any;
    renderMode: number;
    renderOrder?: number;
    speedFactor?: number;
    texture: string;
    startTileIndex: FunctionJSON | number;
    uTileCount: number;
    vTileCount: number;
    blending: number;
    behaviors: Array<any>;
    worldSpace: boolean;
}
export interface BillBoardSettings {
}
export interface TrailSettings {
    startLength: ValueGenerator | FunctionValueGenerator;
    followLocalOrigin: boolean;
}
export interface MeshSettings {
    startRotationX: ValueGenerator | FunctionValueGenerator;
    startRotationY: ValueGenerator | FunctionValueGenerator;
    startRotationZ: ValueGenerator | FunctionValueGenerator;
}
export interface EmissionState {
    burstIndex: number;
    burstWaveIndex: number;
    time: number;
    waitEmiting: number;
}
export declare class ParticleSystem {
    autoDestroy: boolean;
    looping: boolean;
    duration: number;
    maxParticle: number;
    startLife: ValueGenerator | FunctionValueGenerator;
    startSpeed: ValueGenerator | FunctionValueGenerator;
    startRotation: ValueGenerator | FunctionValueGenerator;
    startSize: ValueGenerator | FunctionValueGenerator;
    startColor: ColorGenerator | FunctionColorGenerator;
    startTileIndex: ValueGenerator;
    rendererEmitterSettings: TrailSettings | MeshSettings | BillBoardSettings;
    emissionOverTime: ValueGenerator | FunctionValueGenerator;
    emissionOverDistance: ValueGenerator | FunctionValueGenerator;
    emissionBursts: Array<BurstParameters>;
    onlyUsedByOther: boolean;
    worldSpace: boolean;
    speedFactor: number;
    particleNum: number;
    paused: boolean;
    private emissionState;
    private emitEnded;
    private markForDestroy;
    private normalMatrix;
    behaviors: Array<Behavior>;
    particles: Array<Particle>;
    emitterShape: EmitterShape;
    emitter: ParticleEmitter;
    rendererSettings: ParticleSystemBatchSettings;
    renderer: BatchedParticleRenderer;
    neededToUpdateRender: boolean;
    get texture(): Texture;
    set texture(texture: Texture);
    get uTileCount(): number;
    set uTileCount(u: number);
    get vTileCount(): number;
    set vTileCount(v: number);
    get renderMode(): RenderMode;
    set renderMode(renderMode: RenderMode);
    get renderOrder(): number;
    set renderOrder(renderOrder: number);
    get blending(): Blending;
    set blending(blending: Blending);
    constructor(renderer: BatchedParticleRenderer, parameters: ParticleSystemParameters);
    pause(): void;
    play(): void;
    private spawn;
    endEmit(): void;
    dispose(): void;
    restart(): void;
    private update;
    emit(delta: number, emissionState: EmissionState, emitterMatrix: Matrix4): void;
    toJSON(meta: MetaData): ParticleSystemJSONParameters;
    static fromJSON(json: ParticleSystemJSONParameters, meta: {
        textures: {
            [uuid: string]: Texture;
        };
        geometries: {
            [uuid: string]: BufferGeometry;
        };
    }, dependencies: {
        [uuid: string]: Behavior;
    }, renderer: BatchedParticleRenderer): ParticleSystem;
    addBehavior(behavior: Behavior): void;
    getRendererSettings(): ParticleSystemBatchSettings;
    clone(): ParticleSystem;
}
